program template;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ELSE}
  {$APPTYPE CONSOLE}
{$ENDIF}

{******************************************************************}
(*
  Released as part of the DGLSDK
  Visit us @ http://www.delphigl.com
  Maintained by Jonas Scholz (i0n0s)
  jonasscholz@web.de
-------------------------------------------------------------------
  License
  -------
  Copyright (C) 2006 www.delphigl.com

  This license applies to everything on www.delphigl.com, except where otherwise noted.

  This software is provided 'as-is', without any express or implied warranty.  In no event will the authors 
  be held liable for any damages arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose, including commercial applications, 
  and to alter it and redistribute it freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not claim that you wrote the original 
  software. If you use this software in a product, an acknowledgment in the product documentation would be 
  appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Description
  -----------
  Simple template for the easySDL-framework

  Requirements
  ------------
  - SDL 1.2 (http://www.libsdl.org/download-1.2.php or your distribution)
  - Freepascal 2.0.0 or above (http://www.freepascal.org/download.html)
  - JEDI-SDL 1.0 (http://www.delphi-jedi.org/Jedi:TEAM_SDL_HOME)
  - DGLSDK for Linux (http://www.delphigl.com/)


  History
  -------
  2006-05-01 Added licence (JS)

  Contact
  -------
  We created this source code in our spare time. If you find a bug or just
  wanna say hello, feel free to write to the contributors: 


  JS
    Jonas Scholz (i0n0s)
    email : jonasscholz@web.de

*******************************************************************************)

uses
  dglOpenGL,
  SDL,
  easySDL,
  easySDLFont,
  easySDLTextures,
  SysUtils,
  Shape3DUnit;

{$I ../dglsdk.inc}

Const

   LIGHT_NONE = 0;
   LIGHT_ORDINARY = 1;
   LIGHT_PER_SPOT = 2;
   LIGHT_PER_SPHEREMAP = 3;

//----------------------------------------------------------------------------
// Licht-Einstellungen
//----------------------------------------------------------------------------

   // Light Model
   Ambient_Model  : Array[0..3] Of Single = (0.2, 0.2, 0.2, 1.0);
   Reflect_Model  : Single = 0;

   // Material
   Mat_Specular   : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Mat_Shininess  : Array[0..0] Of Single = (50.0);
   Mat_Ambient    : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Mat_Diffuse    : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);

   // Normal Light
   Light_Position : Array[0..3] Of Single = (-4, 0, 0, 1);
   Light_Ambient  : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);
   Light_Diffuse  : Array[0..3] Of Single = (0.8, 0.8, 0.8, 1.0);

   // Spotlight
   SpotL_Position : Array[0..3] Of Single = (  0,   0,   1, 1);
   SpotL_Direction: Array[0..2] Of Single = (0.4, 0.0,-1.0);
   SpotL_Ambient  : Array[0..3] Of Single = (0.8, 0.2, 0.2, 1.0);
   SpotL_Diffuse  : Array[0..3] Of Single = (0.8, 0.2, 0.2, 1.0);


type
  TMyProgram = class(TEasySDL)
    FPSFont: TFont;
    procedure DrawScene; override;
  end;

var
    MyProgram   : TMyProgram;
    MyShape3D: TShape3D;
    Switch: Byte;
    Angle: Word;
//----------------------------------------------------------------------------
// OpenGL-Einstellungen
//----------------------------------------------------------------------------
Procedure SetOpenGLParams;
Begin
   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);
   glEnable(GL_Texture_2D);
   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);
   glPolygonMode(GL_FRONT,GL_FILL);   // Filled mode
   //glPolygonMode(GL_FRONT,GL_LINE);   // Wireframe mode
   glClearColor(0,0,0,1);
End;
//----------------------------------------------------------------------------
// Demo-Einstellungen
//----------------------------------------------------------------------------
Procedure Settings;
Begin
   // Light Model
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, @Ambient_Model[0]);
   glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, @Reflect_Model);

   // Material
   glMaterialfv(GL_FRONT, GL_SPECULAR,  @Mat_Specular[0]);
   glMaterialfv(GL_FRONT, GL_SHININESS, @Mat_Shininess[0]);
   glMaterialfv(GL_FRONT, GL_AMBIENT,   @Mat_Ambient[0]);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   @Mat_Diffuse[0]);

   // Normal Light
   If Switch = LIGHT_ORDINARY Then Begin
     glLightfv(GL_LIGHT0, GL_AMBIENT,  @Light_Ambient[0]);
     glLightfv(GL_LIGHT0, GL_DIFFUSE,  @Light_Diffuse[0]);
     glLightfv(GL_LIGHT0, GL_POSITION, @Light_Position[0]);

     glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.04);
     glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0);
     glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.01);
   End;

   // Spotlight
   If Switch = LIGHT_PER_SPOT Then Begin
     glLightfv(GL_LIGHT0, GL_POSITION, @SpotL_Position[0]);
     glLightfv(GL_LIGHT0, GL_AMBIENT,  @SpotL_Ambient[0]);
     glLightfv(GL_LIGHT0, GL_DIFFUSE,  @SpotL_Diffuse[0]);

     glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, @SpotL_Direction[0]);
     glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 20.0);
     glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 10.0);
   End;

   // Enable/Disable Light
   If (Switch = LIGHT_ORDINARY) Or (Switch = LIGHT_PER_SPOT)
     Then Begin
       glEnable(GL_COLOR_MATERIAL);
       glEnable(GL_LIGHTING);
       glEnable(GL_LIGHT0);
     End
     Else Begin
       glDisable(GL_COLOR_MATERIAL);
       glDisable(GL_LIGHTING);
       glDisable(GL_LIGHT0);
     End;

   // Enable/Disable Spheremap
   If Switch = LIGHT_PER_SPHEREMAP
     Then Begin
       glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
       glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
       glEnable(GL_TEXTURE_GEN_S);
       glEnable(GL_TEXTURE_GEN_T);
     End
     Else Begin
       glDisable(GL_TEXTURE_GEN_S);
       glDisable(GL_TEXTURE_GEN_T);
     End;
End;
//----------------------------------------------------------------------------
// Zeichnen der Szene
//----------------------------------------------------------------------------
procedure TMyProgram.DrawScene;
begin
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );
  glMatrixMode(GL_PROJECTION);  // Change Matrix Mode to Projection
  glLoadIdentity();             // Reset View
    glOrtho(0, 640, 0, 480, 0, 100);
  glMatrixMode(GL_MODELVIEW);   // Change Projection to Matrix Mode
  glLoadIdentity;
    FPSFont.Draw('Frames per second: '+InttoStr(Round(FPS)),0,0);
    HandleResize(640,480);
  glLoadIdentity;


  If Switch = LIGHT_PER_SPHEREMAP
     Then Textures.Bind('Verlauf')
     Else glBindTexture(GL_TEXTURE_2D,0);

  glColor3f(0,0,1);
  glTranslatef(0,0,-6);

  glPushMatrix;
     glTranslatef(1.5,0,0);
     glRotatef(Angle,0,1,0);
     glRotatef(45,1,0,0);


     MyShape3D.Render(FALSE);   // Zeichnet mit Normalen per Face
  glPopMatrix;

  glTranslatef(-1.5,0,0);
  glRotatef(Angle,0,1,0);
  glRotatef(45,1,0,0);

  MyShape3D.Render(TRUE);

  Inc(Angle);
  If Angle=360
     Then Angle:=0;
end;

(******************************************************************************)
(******************************************************************************)
//  Hauptprogramm
(******************************************************************************)
(******************************************************************************)
var Color: TSDL_Color;

begin
  // Initalisierung
  MyProgram := TMyProgram.Create;
  MyProgram.WindowCaption := 'www.delphigl.com .: Tutorial 8: LICHT';

  SetOpenGLParams;

// Die Variable Switch bestimmt, mit welchem Licht angezeigt wird
  //Switch:= LIGHT_NONE;
  Switch:= LIGHT_ORDINARY;
  //Switch:= LIGHT_PER_SPOT;             
  //Switch:= LIGHT_PER_SPHEREMAP;

  Settings;

  MyShape3D:= TShape3D.Create(stTorus,36,18);
  //MyShape3D:= TShape3D.Create(stSphere,36,18);
  Angle:=0;

  Color.r:=255;
  Color.g:=255;
  Color.b:=255;
  Color.unused:=0;

  MyProgram.FPSFont:=TFont.Create(DGLSDK_DATA+'fonts/monofont.ttf',16, Color, 0);
  Textures.Load('Rotverlauf.bmp','Verlauf');

  // Programmhauptschleife
  MyProgram.Loop;

  // Programmende
  MyShape3D.Free;
  MyProgram.Free;
end.
